/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XHTML + Voice support.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Chris Cross <xcross@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <stdio.h>

#include "nsIGenericFactory.h"
#include "nsIServiceManager.h"
#include "nsIFile.h"
#include "nsString.h"
#include "nsStringAPI.h"

#include "nsVxmlElementFactory.h"
#include "nsVxmlInterpreter.h"

nsString g_ModuleLocation;
nsVxmlInterpreter * g_Interpreter = NULL;

NS_GENERIC_FACTORY_CONSTRUCTOR(nsVxmlElementFactory)

//NS_GENERIC_FACTORY_CONSTRUCTOR(nsVxmlInterpreter)
//Copied the generic ctor and modifying to implement singleton Interpreter

static NS_IMETHODIMP
nsVxmlInterpreterConstructor(nsISupports *aOuter, REFNSIID aIID, void **aResult)
{
  nsresult rv;
  printf("\n*******!!!!!!nsVxmlInterpreterConstructor!!!!!!!*********\n");
  //If the interpreter is already allocated just return it
  if (g_Interpreter) {
    rv = g_Interpreter->QueryInterface(aIID, aResult);
    return rv;
  }

  //The first time through create a singleton interpreter
  *aResult = NULL;
  if (NULL != aOuter) {
    rv = NS_ERROR_NO_AGGREGATION;
    return rv;
  }

  NS_NEWXPCOM(g_Interpreter, nsVxmlInterpreter);
  if (NULL == g_Interpreter) {
    rv = NS_ERROR_OUT_OF_MEMORY;
    return rv;
  } 
  NS_ADDREF(g_Interpreter);
  rv = g_Interpreter->QueryInterface(aIID, aResult);

  //Modify the property descriptor to set the location of the module
  nsCOMPtr<nsIVxmlPropertyDescriptor> pd;
  g_Interpreter->GetPropertyDescriptor(getter_AddRefs(pd));
  PRInt16 rc;
  pd->SetInstallPath(g_ModuleLocation.get(), &rc);
  pd->SetResourcePath(g_ModuleLocation.get(), &rc);

  return rv;
}


static NS_IMETHODIMP
RegisterVxmlModule(nsIComponentManager *aCompMgr,
                     nsIFile *aPath,
                     const char *aRegistryLocation,
                     const char *aComponentType,
                     const nsModuleComponentInfo *aInfo)
{
  printf("\n\n!!!!!!!!!!!!!!!!!!!!!!!!!!RegisterVxmlModule !!!!!!!!!!!!!!!!!!!!\n");
	return NS_OK;
}

static NS_IMETHODIMP
UnregisterVxmlModule(nsIComponentManager *aCompMgr,
                       nsIFile *aPath,
                       const char *aRegistryLocation,
                       const nsModuleComponentInfo *aInfo)
{
	return NS_OK;
}

static const nsModuleComponentInfo components[] = {
  { "X+V element factory",
    NS_VXMLELEMENTFACTORY_CID,
    NS_XTF_ELEMENT_FACTORY_CONTRACTID_PREFIX NS_NAMESPACE_VXML,
    nsVxmlElementFactoryConstructor,
    RegisterVxmlModule,
    UnregisterVxmlModule
  },
  { "X+V Service",
    NS_IVXMLINTERPRETER_IID,
    "@mozilla.org/vxml",
    nsVxmlInterpreterConstructor,
  }
};



//NS_IMPL_NSGETMODULE(xplusv, components)
//Providing our own NSGetModule in order to cache location info for the interpreter
//see http://lxr.mozilla.org/mozilla1.8.0/source/xpcom/glue/nsIGenericFactory.h#332

static nsModuleInfo const kModuleInfo = {
     NS_MODULEINFO_VERSION,
     "xplusv",
     components,
     (sizeof(components) / sizeof(components[0])),
     nsnull,
     nsnull
};


NSGETMODULE_ENTRY_POINT (NSGetModule)(nsIComponentManager* aCompMgr,
                                      nsIFile* aLocation,
                                      nsIModule* *aResult)
{
  //Get our module's location in the file system
  nsresult rv;
  nsCOMPtr<nsIFile> dir;
  rv = aLocation->GetParent(getter_AddRefs(dir));
  if (NS_FAILED(rv))
    return rv;

  //Assume the component will be in "libraries" dir at the same level
  dir->SetNativeLeafName(NS_LITERAL_CSTRING("libraries"));
  nsCAutoString path;
  dir->GetNativePath(path);

  AppendASCIItoUTF16(path, g_ModuleLocation);

  //Continue with XPCOM module initialization
  return NS_NewGenericModule2(&kModuleInfo, aResult);
}

/*
//NS_IMPL_NSGETMODULE_WITH_CTOR(xplusv, components, VxmlModuleCtor)
PR_STATIC_CALLBACK(nsresult)
VxmlModuleCtor(nsIModule* aSelf)
{
  printf("\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!VxmlModuleCtor()\n");
  fflush(stdout);
  return NS_OK;
}
*/
